@@ -1,10 +1,107 @@
 package WordList;
 
-our $DATE = '2018-02-20'; # DATE
-our $VERSION = '0.1.5'; # VERSION
+our $DATE = '2018-03-19'; # DATE
+our $VERSION = '0.2.0'; # VERSION
 
-our @ISA = qw(WordListC);
-use WordListC ();
+use strict 'subs', 'vars';
+
+sub new {
+    my $class = shift;
+    my $fh = \*{"$class\::DATA"};
+    binmode $fh, "encoding(utf8)";
+    unless (defined ${"$class\::DATA_POS"}) {
+        ${"$class\::DATA_POS"} = tell $fh;
+    }
+    bless [], $class;
+}
+
+sub each_word {
+    my ($self, $code) = @_;
+
+    my $class = ref($self);
+
+    my $fh = \*{"$class\::DATA"};
+
+    seek $fh, ${"$class\::DATA_POS"}, 0;
+    while (defined(my $word = <$fh>)) {
+        chomp $word;
+        $code->($word);
+    }
+}
+
+sub pick {
+    my ($self, $n) = @_;
+
+    $n ||= 1;
+
+    my $class = ref($self);
+
+    my $fh = \*{"$class\::DATA"};
+
+    seek $fh, ${"$class\::DATA_POS"}, 0;
+    if ($n < 1) {
+        die "Please specify a positive number of words to pick";
+    } elsif ($n == 1) {
+        # use algorithm from Learning Perl
+        my $word;
+        my $i = 0;
+        while (defined(my $line = <$fh>)) {
+            $i++;
+            $word = $line if rand($i) < 1;
+        }
+        chomp($word);
+        return $word;
+    } else {
+        my @words;
+        my $i = 0;
+        while (defined(my $line = <$fh>)) {
+            $i++;
+            if (@words < $n) {
+                # we haven't reached $n, put word to result in a random position
+                splice @words, rand(@words+1), 0, $line;
+            } else {
+                # we have reached $n, just replace a word randomly, using
+                # algorithm from Learning Perl, slightly modified
+                rand($i) < @words and splice @words, rand(@words), 1, $line;
+            }
+        }
+        chomp(@words);
+        return @words;
+    }
+}
+
+sub word_exists {
+    my ($self, $word) = @_;
+
+    my $class = ref($self);
+
+    my $fh = \*{"$class\::DATA"};
+
+    seek $fh, ${"$class\::DATA_POS"}, 0;
+    while (defined(my $line = <$fh>)) {
+        chomp $line;
+        if ($word eq $line) {
+            return 1;
+        }
+    }
+    0;
+}
+
+sub all_words {
+    my ($self) = @_;
+
+    my $class = ref($self);
+
+    my $fh = \*{"$class\::DATA"};
+
+    seek $fh, ${"$class\::DATA_POS"}, 0;
+    my @res;
+    while (defined(my $word = <$fh>)) {
+        chomp $word;
+        push @res, $word;
+    }
+    @res;
+}
 
 # TODO: binary search method, etc
 
@@ -23,7 +120,7 @@ WordList - Word lists
 
 =head1 VERSION
 
-This document describes version 0.1.5 of WordList (from Perl distribution WordList), released on 2018-02-20.
+This document describes version 0.2.0 of WordList (from Perl distribution WordList), released on 2018-03-19.
 
 =head1 SYNOPSIS
 
@@ -41,13 +138,17 @@ are read-only/immutable and the modules are designed to have low startup
 overhead. This makes them more suitable for use in CLI scripts which often only
 want to pick a word from one or several lists.
 
-Words (or phrases) must be put in C<__DATA__> section, *sorted* ascibetically
-(or by Unicode code point), one per line. Putting the wordlist in the
-C<__DATA__> section relieves perl from having to parse the list during the
-loading of the module. To search for words or picking some random words from the
-list, the module also need not slurp the whole list into memory (and will not do
-so unless explicitly instructed). Sorting makes it more convenient to diff
-different versions of the module, as well as performing binary search.
+Words (or phrases) must be put in C<__DATA__> section, one per line. Putting the
+wordlist in the C<__DATA__> section relieves perl from having to parse the list
+during the loading of the module. To search for words or picking some random
+words from the list, the module also need not slurp the whole list into memory
+(and will not do so unless explicitly instructed).
+
+You must sort your words ascibetically (or by Unicode code point). Sorting makes
+it more convenient to diff different versions of the module, as well as
+performing binary search. If you have a different sort order other than
+ascibetical, you must set package variable C<$SORT> with some true value (say,
+C<frequency>).
 
 Since this is a new and non-backward compatible interface from
 Games::Word::Wordlist, I also make some other changes:
@@ -138,9 +239,6 @@ feature.
 
 =head1 SEE ALSO
 
-L<WordListC> is just like L<WordList> except it does not impose ascibetical
-sorting order requirement. You can sort the wordlist in whatever order you need.
-
 L<Bencher::Scenario::GamesWordlistModules>
 
 L<Bencher::Scenario::WordListModules>
